"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es6.regexp.to-string");

var _hex = _interopRequireDefault(require("@polkadot/util/is/hex"));

var _u8a = _interopRequireDefault(require("@polkadot/util/is/u8a"));

var _concat = _interopRequireDefault(require("@polkadot/util/u8a/concat"));

var _toU8a = _interopRequireDefault(require("@polkadot/util/u8a/toU8a"));

var _Base = _interopRequireDefault(require("./codec/Base"));

var _AccountId = _interopRequireDefault(require("./AccountId"));

var _AccountIndex = _interopRequireDefault(require("./AccountIndex"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// A wrapper around an AccountId and/or AccountIndex that is encoded with a prefix.
// Since we are dealing with underlying publicKeys (or shorter encoded addresses),
// we extend from Base with an AccountId/AccountIndex wrapper. Basically the Address
// is encoded as
//   [ <prefix-byte>, ...publicKey/...bytes ]
class Address extends _Base.default {
  constructor(value = new Uint8Array()) {
    super(value instanceof Address ? value.raw : Address.decode(value));
  }

  static decode(value) {
    if (value instanceof _AccountId.default || value instanceof _AccountIndex.default) {
      return value;
    } else if ((0, _u8a.default)(value) || Array.isArray(value)) {
      return value.length === 32 ? new _AccountId.default((0, _toU8a.default)(value)) : new _AccountIndex.default((0, _toU8a.default)(value));
    } else if ((0, _hex.default)(value)) {
      return value.length === 66 ? new _AccountId.default(value) : new _AccountIndex.default(value);
    } // FIXME This _may_ be an issue. Depending on how the actual AccountIndex is
    // actually encoded as a string. Here we just assume that it is an ss-58 value
    // and use the stock-stand AccountId as PublicKey


    return new _AccountId.default(value);
  } // TODO
  //   - Double-check this logic again - if really <= 0xef, the throw
  //     is unneeded since all cases are catered for here
  //   - We probably want to clean the read/write up with enums if it
  //     is an exact check and not a <= check (first item)


  static readLength(input) {
    const first = input[0];

    if (first <= 0xef) {
      return 1;
    } else if (first === 0xfc) {
      return 2;
    } else if (first === 0xfd) {
      return 4;
    } else if (first === 0xfe) {
      return 8;
    } else if (first === 0xff) {
      return 32;
    }

    throw new Error(`Invalid account index byte, 0x${first.toString(16)}`);
  }

  static writeLength(length) {
    if (length === 1) {
      return new Uint8Array([0xef]);
    } else if (length === 2) {
      return new Uint8Array([0xfc]);
    } else if (length === 4) {
      return new Uint8Array([0xfd]);
    } else if (length === 8) {
      return new Uint8Array([0xfe]);
    } else if (length === 32) {
      return new Uint8Array([0xff]);
    }

    throw new Error(`Invalid bitLength, ${length * 8}`);
  }

  byteLength() {
    return 1 + super.byteLength();
  }

  fromJSON(input) {
    this.raw = Address.decode(input);
    return this;
  } // FIXME I actually believe this is wrong, it needs to be checked. Here 1-byte
  // addresses are actually 2 bytes, 2-byte is 3-bytes, etc. I believe the actual
  // short-encoding should be left completely inside AccountId (For now, leaving
  // as-is until we can actually pull storage data and check - and/or read the
  // actual code again to check the implementation)


  fromU8a(input) {
    this.raw = Address.readLength(input) === 32 ? new _AccountId.default().fromU8a(input.subarray(1)) : new _AccountIndex.default().fromU8a(input.subarray(1));
    return this;
  }

  toJSON() {
    return this.raw.toJSON();
  }

  toString() {
    return this.raw.toString();
  }

  toU8a(isBare) {
    return isBare ? this.raw.toU8a(true) : (0, _concat.default)(Address.writeLength(this.raw.byteLength()), this.raw.toU8a(isBare));
  }

}

exports.default = Address;