"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Justification = exports.BftHashSignature = exports.BftAuthoritySignature = void 0;

require("core-js/modules/web.dom.iterable");

var _Struct = _interopRequireDefault(require("./codec/Struct"));

var _Tuple = _interopRequireDefault(require("./codec/Tuple"));

var _Vector = _interopRequireDefault(require("./codec/Vector"));

var _AuthorityId = _interopRequireDefault(require("./AuthorityId"));

var _Hash = _interopRequireDefault(require("./Hash"));

var _Signature = _interopRequireDefault(require("./Signature"));

var _U = _interopRequireDefault(require("./U32"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// Represents a Bft Hash and Signature pairing, typically used in reporting
// network behaviour.
class BftAuthoritySignature extends _Tuple.default {
  constructor(value) {
    super({
      authorityId: _AuthorityId.default,
      signature: _Signature.default
    }, value);
  }

  get authorityId() {
    return this.raw.authorityId;
  }

  get signature() {
    return this.raw.signature;
  }

}

exports.BftAuthoritySignature = BftAuthoritySignature;

// Represents a Bft Hash and Signature pairing, typically used in reporting
// network behaviour.
class BftHashSignature extends _Tuple.default {
  constructor(value) {
    super({
      hash: _Hash.default,
      signature: _Signature.default
    }, value);
  }

  get hash() {
    return this.raw.hash;
  }

  get signature() {
    return this.raw.signature;
  }

}

exports.BftHashSignature = BftHashSignature;

class Justification extends _Struct.default {
  constructor(value) {
    super({
      // FIXME Rust returns this as "round_number", we actually want a JSON alias
      // in the structure to take care of these renames...
      round: _U.default,
      hash: _Hash.default,
      signatures: _Vector.default.with(BftAuthoritySignature)
    }, value, new Map([['round', 'round_number']]));
  }

  get hash() {
    return this.raw.hash;
  }

  get round() {
    return this.raw.round;
  }

  get signatures() {
    return this.raw.signatures;
  }

}

exports.Justification = Justification;