"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.Digest = void 0;

var _asU8a = _interopRequireDefault(require("@polkadot/util-crypto/blake2/asU8a"));

var _Struct = _interopRequireDefault(require("./codec/Struct"));

var _Vector = _interopRequireDefault(require("./codec/Vector"));

var _BlockNumber = _interopRequireDefault(require("./BlockNumber"));

var _Bytes = _interopRequireDefault(require("./Bytes"));

var _Hash = _interopRequireDefault(require("./Hash"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// A block header digest.
class Digest extends _Struct.default {
  constructor(value) {
    super({
      logs: _Vector.default.with(_Bytes.default)
    }, value);
  }

  get logs() {
    return this.raw.logs;
  }

} // A block header.


exports.Digest = Digest;

class Header extends _Struct.default {
  constructor(value) {
    super({
      parentHash: _Hash.default,
      number: _BlockNumber.default,
      stateRoot: _Hash.default,
      extrinsicsRoot: _Hash.default,
      digest: Digest
    }, value);
  }

  get blockNumber() {
    return this.raw.number;
  }

  get digest() {
    return this.raw.digest;
  }

  get extrinsicsRoot() {
    return this.raw.extrinsicsRoot;
  } // convenience, encodes the header and returns the actual hash


  get hash() {
    return new _Hash.default((0, _asU8a.default)(this.toU8a(), 256));
  }

  get parentHash() {
    return this.raw.parentHash;
  }

  get stateRoot() {
    return this.raw.stateRoot;
  }

}

exports.default = Header;