"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.KeyValueOption = exports.default = void 0;

var _Option = _interopRequireDefault(require("./codec/Option"));

var _Struct = _interopRequireDefault(require("./codec/Struct"));

var _Tuple = _interopRequireDefault(require("./codec/Tuple"));

var _StorageData = _interopRequireDefault(require("./StorageData"));

var _StorageKey = _interopRequireDefault(require("./StorageKey"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// KeyValue structure. Since most of the keys and resultant values in Subtrate is
// hashed and/or encoded, this does not wrap a Text, but rather a Bytes
// for the keys and values. (Not to be confused with the KeyValue in Metadata, that
// is actually for Maps, whereas this is a representation of actaul storage values)
class KeyValue extends _Struct.default {
  constructor(value) {
    super({
      key: _StorageKey.default,
      value: _StorageData.default
    }, value);
  }

  get key() {
    return this.raw.key;
  }

  get value() {
    return this.raw.value;
  }

}

exports.default = KeyValue;

// A key/value change. This is similar to the KeyValue structure,
// however in this case the value could be optional. Here it extends
// from a Tuple, indicating the use inside areas such as StorageChangeSet
class KeyValueOption extends _Tuple.default {
  constructor(value) {
    super({
      key: _StorageKey.default,
      value: _Option.default.with(_StorageData.default)
    }, value);
  }

  get key() {
    return this.raw.key;
  }

  get value() {
    return this.raw.value;
  }

}

exports.KeyValueOption = KeyValueOption;