"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.RuntimeModuleMetadata = exports.StorageMetadata = exports.StorageFunctionMetadata = exports.StorageFunctionType = exports.StorageFunctionType$Map = exports.StorageFunctionModifier = exports.ModuleMetadata = exports.CallMetadata = exports.FunctionMetadata = exports.FunctionArgumentMetadata = exports.OuterEventMetadata = exports.OuterEventMetadataEvent = exports.EventMetadata = void 0;

require("core-js/modules/web.dom.iterable");

require("core-js/modules/es6.array.sort");

require("core-js/modules/es6.regexp.to-string");

var _toU8a = _interopRequireDefault(require("@polkadot/util/u8a/toU8a"));

var _Vector = _interopRequireDefault(require("./codec/Vector"));

var _Enum = _interopRequireDefault(require("./codec/Enum"));

var _EnumType = _interopRequireDefault(require("./codec/EnumType"));

var _Option = _interopRequireDefault(require("./codec/Option"));

var _Struct = _interopRequireDefault(require("./codec/Struct"));

var _Text = _interopRequireDefault(require("./Text"));

var _Type = _interopRequireDefault(require("./Type"));

var _U = _interopRequireDefault(require("./U16"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// Decodes the runtime metadata as passed through from the `state_getMetadata` call. This
// file is probably best understood from the bottom-up, i.e. start reading right at the
// end and work up. (Just so we don't use before definition)
class EventMetadata extends _Struct.default {
  constructor(value) {
    super({
      name: _Text.default,
      arguments: _Vector.default.with(_Type.default),
      documentation: _Vector.default.with(_Text.default)
    }, value);
  }

  get arguments() {
    return this.raw.arguments;
  }

  get documentation() {
    return this.raw.documentation;
  }

  get name() {
    return this.raw.name;
  }

}

exports.EventMetadata = EventMetadata;

class OuterEventMetadataEvent extends _Struct.default {
  constructor(value) {
    super({
      name: _Text.default,
      events: _Vector.default.with(EventMetadata)
    }, value);
  }

  get events() {
    return this.raw.events;
  }

  get name() {
    return this.raw.name;
  }

}

exports.OuterEventMetadataEvent = OuterEventMetadataEvent;

class OuterEventMetadata extends _Struct.default {
  constructor(value) {
    super({
      name: _Text.default,
      events: _Vector.default.with(OuterEventMetadataEvent)
    }, value);
  }

  get events() {
    return this.raw.events;
  }

  get name() {
    return this.raw.name;
  }

}

exports.OuterEventMetadata = OuterEventMetadata;

class FunctionArgumentMetadata extends _Struct.default {
  constructor(value) {
    super({
      name: _Text.default,
      type: _Type.default
    }, value);
  }

  get name() {
    return this.raw.name;
  }

  get type() {
    return this.raw.type;
  }

}

exports.FunctionArgumentMetadata = FunctionArgumentMetadata;

class FunctionMetadata extends _Struct.default {
  constructor(value) {
    super({
      id: _U.default,
      name: _Text.default,
      arguments: _Vector.default.with(FunctionArgumentMetadata),
      documentation: _Vector.default.with(_Text.default)
    }, value);
  }

  get arguments() {
    return this.raw.arguments;
  }

  get documentation() {
    return this.raw.documentation;
  }

  get id() {
    return this.raw.id;
  }

  get name() {
    return this.raw.name;
  }

}

exports.FunctionMetadata = FunctionMetadata;

class CallMetadata extends _Struct.default {
  constructor(value) {
    super({
      name: _Text.default,
      functions: _Vector.default.with(FunctionMetadata)
    }, value);
  }

  get functions() {
    return this.raw.functions;
  }

  get name() {
    return this.raw.name;
  }

}

exports.CallMetadata = CallMetadata;

class ModuleMetadata extends _Struct.default {
  constructor(value) {
    super({
      name: _Text.default,
      call: CallMetadata
    }, value);
  }

  get call() {
    return this.raw.call;
  }

  get name() {
    return this.raw.name;
  }

}

exports.ModuleMetadata = ModuleMetadata;

class StorageFunctionModifier extends _Enum.default {
  constructor(value) {
    super(['None', 'Default', 'Required'], value);
  }

}

exports.StorageFunctionModifier = StorageFunctionModifier;

class StorageFunctionType$Map extends _Struct.default {
  constructor(value) {
    super({
      key: _Type.default,
      value: _Type.default
    }, value);
  }

  get key() {
    return this.raw.type;
  }

  get value() {
    return this.raw.value;
  }

}

exports.StorageFunctionType$Map = StorageFunctionType$Map;

class StorageFunctionType extends _EnumType.default {
  constructor(index, value) {
    super([_Type.default, StorageFunctionType$Map], index, value);
  }

  get isMap() {
    return this.toNumber() === 1;
  }

  get asMap() {
    return this.raw.raw;
  }

  get asType() {
    return this.raw.raw;
  }

  toString() {
    return this.isMap ? this.asMap.value.toString() : this.asType.toString();
  }

}

exports.StorageFunctionType = StorageFunctionType;

class StorageFunctionMetadata extends _Struct.default {
  constructor(value) {
    super({
      name: _Text.default,
      modifier: StorageFunctionModifier,
      type: StorageFunctionType,
      documentation: _Vector.default.with(_Text.default)
    }, value);
  }

  get documentation() {
    return this.raw.documentation;
  }

  get name() {
    return this.raw.name;
  }

  get modifier() {
    return this.raw.modifier;
  }

  get type() {
    return this.raw.type;
  }

}

exports.StorageFunctionMetadata = StorageFunctionMetadata;

class StorageMetadata extends _Struct.default {
  constructor(value) {
    super({
      prefix: _Text.default,
      functions: _Vector.default.with(StorageFunctionMetadata)
    }, value);
  }

  get functions() {
    return this.raw.functions;
  }

  get prefix() {
    return this.raw.prefix;
  }

}

exports.StorageMetadata = StorageMetadata;

class RuntimeModuleMetadata extends _Struct.default {
  constructor(value) {
    super({
      prefix: _Text.default,
      module: ModuleMetadata,
      storage: _Option.default.with(StorageMetadata)
    }, value);
  }

  get module() {
    return this.raw.module;
  }

  get prefix() {
    return this.raw.prefix;
  }

  get storage() {
    return this.raw.storage.value;
  }

}

exports.RuntimeModuleMetadata = RuntimeModuleMetadata;

class RuntimeMetadata extends _Struct.default {
  constructor(value) {
    super({
      outerEvent: OuterEventMetadata,
      modules: _Vector.default.with(RuntimeModuleMetadata)
    }, value);
  } // We receive this as an Array<number> in the JSON output from the Node. Convert
  // to u8a and use the fromU8a to do the actual parsing


  fromJSON(input) {
    return this.fromU8a((0, _toU8a.default)(input));
  } // FIXME Currently toJSON creates a struct, so it is not a one-to-one mapping
  // with what is actually found on the RPC layer. This needs to be adjusted to
  // match fromJSON. (However for now, it is useful in debugging)


  toJSON() {
    return super.toJSON();
  }

  get events() {
    return this.raw.outerEvent.events;
  }

  get modules() {
    return this.raw.modules;
  } // Helper to retrieve a list of all type that are found, sorted and de-deuplicated


  getUniqTypes() {
    // Quick and dirty flatten (.flat() not available)
    const flatten = list => list.reduce((result, entry) => {
      return result.concat(Array.isArray(entry) ? flatten(entry) : entry);
    }, []);

    const events = this.events.map(module => module.events.map(event => event.arguments.map(argument => argument.raw)));
    const storages = this.modules.map(module => module.storage ? module.storage.functions.map(fn => fn.type.isMap ? [fn.type.asMap.key.raw, fn.type.asMap.value.raw] : [fn.type.asType.raw]) : []);
    const args = this.modules.map(module => module.module.call.functions.map(fn => fn.arguments.map(argument => argument.type.raw)));
    return [...new Set(flatten([events, storages, args]).filter(value => value))].sort();
  }

}

exports.default = RuntimeMetadata;