"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.MisbehaviorKind = exports.BftDoubleCommit = exports.BftDoublePrepare = exports.BftAtReport = void 0;

var _EnumType = _interopRequireDefault(require("./codec/EnumType"));

var _Struct = _interopRequireDefault(require("./codec/Struct"));

var _AuthorityId = _interopRequireDefault(require("./AuthorityId"));

var _Bft = require("./Bft");

var _BlockNumber = _interopRequireDefault(require("./BlockNumber"));

var _Hash = _interopRequireDefault(require("./Hash"));

var _U = _interopRequireDefault(require("./U32"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// A report of a/b hash-signature pairs for a specific index. This is the same
// structure as is used in BftDoublePrepare & BftDoubleCommit
//
// FIXME It is not entirely obvious from the actual Rust code what the specific
// items in the structure is called, except a & b (one should be expected, the
// other actual)
class BftAtReport extends _Struct.default {
  constructor(value) {
    super({
      round: _U.default,
      a: _Bft.BftHashSignature,
      b: _Bft.BftHashSignature
    }, value);
  }

  get a() {
    return this.raw.a;
  }

  get b() {
    return this.raw.b;
  }

  get round() {
    return this.raw.round;
  }

} // Report of a double-prepare


exports.BftAtReport = BftAtReport;

class BftDoublePrepare extends BftAtReport {} // Report of a double-commit


exports.BftDoublePrepare = BftDoublePrepare;

class BftDoubleCommit extends BftAtReport {}

exports.BftDoubleCommit = BftDoubleCommit;

class MisbehaviorKind extends _EnumType.default {
  constructor(index = 0x11, value) {
    super({
      0x11: BftDoublePrepare,
      0x12: BftDoubleCommit
    }, index, value);
  }

}

exports.MisbehaviorKind = MisbehaviorKind;

// A Misbehaviour report against a specific AuthorityId
class MisbehaviorReport extends _Struct.default {
  constructor(value) {
    super({
      parentHash: _Hash.default,
      parentNumber: _BlockNumber.default,
      target: _AuthorityId.default,
      misbehavior: MisbehaviorKind
    }, value);
  }

  get misbehavior() {
    return this.raw.misbehavior;
  }

  get parentHash() {
    return this.raw.parentHash;
  }

  get parentNumber() {
    return this.raw.parentNumber;
  }

  get target() {
    return this.raw.target;
  }

}

exports.default = MisbehaviorReport;