"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _bn = _interopRequireDefault(require("bn.js"));

var _toBn = _interopRequireDefault(require("@polkadot/util/bn/toBn"));

var _toU8a = _interopRequireDefault(require("@polkadot/util/bn/toU8a"));

var _u8a = _interopRequireDefault(require("@polkadot/util/is/u8a"));

var _toBn2 = _interopRequireDefault(require("@polkadot/util/u8a/toBn"));

var _Base = _interopRequireDefault(require("./codec/Base"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
const BITLENGTH = 64; // A wrapper around seconds/timestamps. Internally the representation only has
// second precicion (aligning with Rust), so any numbers passed an/out are always
// per-second. For any encoding/decoding the 1000 multiplier would be applied to
// get it in line with JavaScript formats

class Moment extends _Base.default {
  constructor(value = 0) {
    super(value instanceof Date ? new Date(Math.ceil(value.getTime() / 1000) * 1000) : Moment.decode(value));
  }

  static decode(value) {
    if (value instanceof Moment) {
      return value.raw;
    }

    if ((0, _u8a.default)(value)) {
      value = (0, _toBn2.default)(value, true);
    }

    return new Date((0, _toBn.default)(value).toNumber() * 1000);
  }

  byteLength() {
    return BITLENGTH / 8;
  }

  fromJSON(input) {
    this.raw = Moment.decode(input);
    return this;
  }

  fromU8a(input) {
    this.raw = Moment.decode((0, _toBn2.default)(input.subarray(0, this.byteLength()), true));
    return this;
  }

  getTime() {
    return this.raw.getTime();
  }

  toJSON() {
    return this.toNumber();
  }

  toU8a(isBare) {
    return (0, _toU8a.default)(this.toNumber(), BITLENGTH, true);
  }

  toString() {
    return this.raw.toISOString();
  }

  toBn() {
    return new _bn.default(this.toNumber());
  }

  toNumber() {
    return Math.ceil(this.raw.getTime() / 1000);
  }

}

exports.default = Moment;