"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es6.regexp.to-string");

var _fromUtf = _interopRequireDefault(require("@polkadot/util/u8a/fromUtf8"));

var _toUtf = _interopRequireDefault(require("@polkadot/util/u8a/toUtf8"));

var _concat = _interopRequireDefault(require("@polkadot/util/u8a/concat"));

var _Base = _interopRequireDefault(require("./codec/Base"));

var _Length = _interopRequireDefault(require("./codec/Length"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// This is a string wrapper, along with the length. It is used both for strings as well
// as stuff like documentation.
//
// TODO
//   - Strings should probably be trimmed (docs do come through with extra padding)
//   - Potentially we want a "TypeString" extension to this. Basically something that
//     wraps the `Balance`, `T::AccountId`, etc. The reasoning - with a "TypeString"
//     we can nicely strip types down like "T::AcountId" -> "AccountId"
class Text extends _Base.default {
  constructor(value = '') {
    super(value instanceof Text ? value.raw : value.trim());
    this._length = void 0;
    this._length = new _Length.default(value.length);
  }

  get length() {
    return this.raw.length;
  }

  byteLength() {
    // NOTE String gets trimmed in the fromU8a, so get the original length from the length
    // placeholder and the associtated byteLength from that encoding
    return this._length.toNumber() + this._length.byteLength();
  }

  fromJSON(input) {
    this.raw = `${input || ''}`;
    return this;
  }

  fromU8a(input) {
    this._length.fromU8a(input);

    const length = this._length.toNumber();

    const offset = this._length.byteLength();

    this.raw = (0, _toUtf.default)(input.subarray(offset, offset + length)).trim();
    return this;
  }

  toJSON() {
    return this.toString();
  }

  toString() {
    return this.raw;
  }

  toU8a(isBare) {
    // NOTE Since we trim in the fromU8a, here we re-calculate the actual length
    const encoded = (0, _fromUtf.default)(this.raw);

    this._length.setValue(encoded.length);

    return (0, _concat.default)(this._length.toU8a(isBare), encoded);
  }

}

exports.default = Text;