"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _bn = _interopRequireDefault(require("bn.js"));

var _toU8a = _interopRequireDefault(require("@polkadot/util/bn/toU8a"));

var _concat = _interopRequireDefault(require("@polkadot/util/u8a/concat"));

var _toBn = _interopRequireDefault(require("@polkadot/util/u8a/toBn"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
const MAX_U8 = new _bn.default(2).pow(new _bn.default(8 - 2)).subn(1);
const MAX_U16 = new _bn.default(2).pow(new _bn.default(16 - 2)).subn(1);
const MAX_U32 = new _bn.default(2).pow(new _bn.default(32 - 2)).subn(1);

// A new compact length-encoding algorithm. It performs the same function as Length, however
// differs in that it uses a variable number of bytes to do the actual encoding. From the Rust
// implementation for compact encoding
//
//     0b00 00 00 00 / 00 00 00 00 / 00 00 00 00 / 00 00 00 00
// (0 ... 2**6 - 1)    (u8)
//     xx xx xx 00
// (2**6 ... 2**14 - 1)  (u8, u16)  low LH high
//     yL yL yL 01 / yH yH yH yL
// (2**14 ... 2**30 - 1)  (u16, u32)  low LMMH high
//     zL zL zL 10 / zM zM zM zL / zM zM zM zM / zH zH zH zM
// (2**30 ... 2**536 - 1)  (u32, u64, u128, U256, U512, U520) straight LE-encoded
//     nn nn nn 11 [ / zz zz zz zz ]{4 + n}
//
// Note: we use *LOW BITS* of the LSB in LE encoding to encode the 2 bit key.
class Compact {
  static decode(input, bitLength) {
    const flag = input[0] & 0b11;

    if (flag === 0b00) {
      return new _bn.default(input[0]).shrn(2);
    } else if (flag === 0b01) {
      return (0, _toBn.default)(input.slice(0, 2), true).shrn(2);
    } else if (flag === 0b10) {
      return (0, _toBn.default)(input.slice(0, 4), true).shrn(2);
    }

    const byteLength = bitLength / 8;
    return (0, _toBn.default)(input.subarray(1, 1 + byteLength), true);
  }

  static encode(length, bitLength) {
    if (length.lte(MAX_U8)) {
      return new Uint8Array([length.toNumber() << 2]);
    } else if (length.lte(MAX_U16)) {
      return (0, _toU8a.default)(length.shln(2).addn(0b01), 16, true);
    } else if (length.lte(MAX_U32)) {
      return (0, _toU8a.default)(length.shln(2).addn(0b10), 32, true);
    }

    return (0, _concat.default)(new Uint8Array([0b11]), (0, _toU8a.default)(length, bitLength, true));
  }

}

exports.default = Compact;