"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es6.regexp.to-string");

var _undefined = _interopRequireDefault(require("@polkadot/util/is/undefined"));

var _Base = _interopRequireDefault(require("./Base"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// An Option is an optional field. Basically the first byte indicates that there is
// is value to follow. If the byte is `1` there is an actual value. So the Option
// implements that - decodes, checks for optionality and wraps the required structure
// with a value if/as required/found.
class Option extends _Base.default {
  constructor(Value, value) {
    super(new Value(value));
    this._isEmpty = void 0;
    this._isEmpty = (0, _undefined.default)(value);
  }

  static with(Type) {
    return class extends Option {
      constructor(value) {
        super(Type, value);
      }

    };
  }

  get isEmpty() {
    return this._isEmpty;
  }

  get value() {
    return this._isEmpty ? undefined : this.raw.raw;
  }

  byteLength() {
    const childLength = this._isEmpty ? 0 : this.raw.byteLength();
    return 1 + childLength;
  }

  fromJSON(input) {
    this._isEmpty = (0, _undefined.default)(input);

    if (!this._isEmpty) {
      this.raw.fromJSON(input);
    }

    return this;
  }

  fromU8a(input) {
    this._isEmpty = input[0] === 0;

    if (!this._isEmpty) {
      this.raw.fromU8a(input.subarray(1));
    }

    return this;
  }

  toJSON() {
    return this._isEmpty ? undefined : this.raw.toJSON();
  }

  toU8a(isBare) {
    if (isBare) {
      return this.raw.toU8a(true);
    }

    const u8a = new Uint8Array(this.byteLength());

    if (!this._isEmpty) {
      u8a.set([1]);
      u8a.set(this.raw.toU8a(), 1);
    }

    return u8a;
  }

  toString() {
    return this._isEmpty ? '' : this.raw.toString();
  }

}

exports.default = Option;