"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es6.regexp.to-string");

require("core-js/modules/web.dom.iterable");

var _toU8a = _interopRequireDefault(require("@polkadot/util/hex/toU8a"));

var _hex = _interopRequireDefault(require("@polkadot/util/is/hex"));

var _concat = _interopRequireDefault(require("@polkadot/util/u8a/concat"));

var _Base = _interopRequireDefault(require("./Base"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// A Struct defines an Object with key/values - where the values are Base<T> values. It removes
// a lot of repetition from the actual coding, define a structure type, pass it the key/Base<T>
// values in the constructor and it manages the decoding. It is important that the constructor
// values matches 100% to the order in th Rust code, i.e. don't go crazy and make it alphabetical,
// it needs to decoded in the specific defined order.
class Struct extends _Base.default {
  constructor(Types, value = {}, jsonMap = new Map(), isTuple = false) {
    super(Struct.decode(Types, value, isTuple));
    this._jsonMap = void 0;
    this._Types = void 0;
    this._jsonMap = jsonMap;
    this._Types = Object.keys(Types).reduce((result, key) => {
      // @ts-ignore Same as above, can't do a simple one, I'm missing something simple
      result[key] = Types[key].name;
      return result;
    }, {});
  }

  static decode(Types, value, isTuple) {
    // l.debug(() => ['Struct.decode', { Types, value }]);
    return Object.keys(Types).reduce((raw, key, index) => {
      // @ts-ignore FIXME Ok, something weird is going on here or I just don't get it...
      // it works, so ignore the checker, although it drives me batty. (It started when
      // the [key in keyof T] was added, the idea is to provide better checks, which
      // does backfire here, but works externally.)
      raw[key] = new Types[key](isTuple && Array.isArray(value) ? value[index] // @ts-ignore as above
      : value[key]);
      return raw;
    }, {});
  }

  static with(Types) {
    return class extends Struct {
      constructor(value, jsonMap) {
        super(Types, value, jsonMap);
      }

    };
  }

  get Type() {
    return this._Types;
  }

  byteLength() {
    return Object.values(this.raw).reduce((length, entry) => {
      return length += entry.byteLength();
    }, 0);
  }

  fromJSON(input) {
    if ((0, _hex.default)(input)) {
      return this.fromU8a((0, _toU8a.default)(input));
    }

    Object.keys(this.raw).forEach(key => {
      const jsonKey = this._jsonMap.get(key) || key; // @ts-ignore as above...

      this.raw[key].fromJSON(input[jsonKey]);
    });
    return this;
  }

  fromU8a(input) {
    Object.values(this.raw).reduce((offset, entry) => {
      entry.fromU8a(input.subarray(offset));
      return offset + entry.byteLength();
    }, 0);
    return this;
  }

  get(index) {
    return this.values()[index];
  }

  toJSON() {
    return Object.keys(this.raw).reduce((json, key) => {
      const jsonKey = this._jsonMap.get(key) || key; // @ts-ignore as above...

      json[jsonKey] = this.raw[key].toJSON();
      return json;
    }, {});
  }

  keys() {
    return Object.keys(this.raw);
  }

  toU8a(isBare) {
    return (0, _concat.default)(...Object.values(this.raw).map(entry => entry.toU8a(isBare)));
  }

  toString() {
    const data = Object.keys(this.raw).map(key => // @ts-ignore as above...
    `${key}: ${this.raw[key].toString()}`).join(', ');
    return `{${data}}`;
  }

  values() {
    return Object.values(this.raw);
  }

}

exports.default = Struct;