"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es6.regexp.to-string");

var _bn = _interopRequireDefault(require("bn.js"));

var _hex = _interopRequireDefault(require("@polkadot/util/is/hex"));

var _string = _interopRequireDefault(require("@polkadot/util/is/string"));

var _u8a = _interopRequireDefault(require("@polkadot/util/is/u8a"));

var _toBn = _interopRequireDefault(require("@polkadot/util/bn/toBn"));

var _toHex = _interopRequireDefault(require("@polkadot/util/bn/toHex"));

var _toU8a = _interopRequireDefault(require("@polkadot/util/bn/toU8a"));

var _toBn2 = _interopRequireDefault(require("@polkadot/util/hex/toBn"));

var _toBn3 = _interopRequireDefault(require("@polkadot/util/u8a/toBn"));

var _Base = _interopRequireDefault(require("./Base"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// A generic number codec. For Substrate all numbers are LE encoded, this handles the encoding
// and decoding of those numbers. Upon construction the bitLength is provided and any additional
// use keeps the number to this length.
//
// TODO:
//   - Apart from encoding/decoding we don't actuall keep check on the sizes, is this good enough?
class UInt extends _Base.default {
  constructor(value = 0, bitLength = 64) {
    super(UInt.decode(value));
    this._bitLength = void 0;
    this._bitLength = bitLength;
  }

  static decode(value) {
    if (value instanceof UInt) {
      return value.raw;
    } else if ((0, _hex.default)(value)) {
      return (0, _toBn2.default)(value);
    } else if ((0, _u8a.default)(value)) {
      // NOTE When passing u8a in (typically from decoded data), it is always u8a
      return (0, _toBn3.default)(value, true);
    } else if ((0, _string.default)(value)) {
      return new _bn.default(value, 10);
    }

    return (0, _toBn.default)(value);
  }

  byteLength() {
    return this._bitLength / 8;
  }

  fromJSON(input) {
    this.raw = UInt.decode(input);
    return this;
  }

  fromU8a(input) {
    this.raw = (0, _toBn3.default)(input.subarray(0, this.byteLength()), true);
    return this;
  }

  toHex() {
    return (0, _toHex.default)(this.raw, this._bitLength);
  }

  toJSON() {
    return this.toHex();
  }

  toU8a(isBare) {
    return (0, _toU8a.default)(this.raw, this._bitLength, true);
  }

  toString() {
    return this.raw.toString();
  }

  toBn() {
    return this.raw;
  }

  toNumber() {
    return this.raw.toNumber();
  } // helpers from BN, this would be great as a "don't do this", i.e. extending properly
  // from BN. Underlying these will always return BN (unless it is compare checks)


  add(other) {
    return other instanceof UInt ? this.raw.add(other.raw) : this.raw.add((0, _toBn.default)(other));
  }

  cmp(other) {
    return other instanceof UInt ? this.raw.cmp(other.raw) : this.raw.cmp((0, _toBn.default)(other));
  }

  div(other) {
    return other instanceof UInt ? this.raw.div(other.raw) : this.raw.div((0, _toBn.default)(other));
  }

  isZero() {
    return this.raw.isZero();
  }

  lt(test) {
    return test instanceof UInt ? this.raw.lt(test.raw) : this.raw.lt((0, _toBn.default)(test));
  }

  lte(test) {
    return test instanceof UInt ? this.raw.lte(test.raw) : this.raw.lte((0, _toBn.default)(test));
  }

  gt(test) {
    return test instanceof UInt ? this.raw.gt(test.raw) : this.raw.gt((0, _toBn.default)(test));
  }

  gte(test) {
    return test instanceof UInt ? this.raw.gte(test.raw) : this.raw.gte((0, _toBn.default)(test));
  }

  mul(other) {
    return other instanceof UInt ? this.raw.mul(other.raw) : this.raw.mul((0, _toBn.default)(other));
  }

  sub(other) {
    return other instanceof UInt ? this.raw.sub(other.raw) : this.raw.sub((0, _toBn.default)(other));
  }

}

exports.default = UInt;