"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es6.regexp.to-string");

require("core-js/modules/web.dom.iterable");

var _concat = _interopRequireDefault(require("@polkadot/util/u8a/concat"));

var _Base = _interopRequireDefault(require("./Base"));

var _Length = _interopRequireDefault(require("./Length"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// This manages codec arrays. Intrernally it keeps track of the length (as decoded) and allows
// construction with the passed `Type` in the constructor. It aims to be an array-like structure,
// i.e. while it wraps an array, it provides a `length` property to get the actual length, `at(index)`
// to retrieve a specific item. Additionally the helper functions `map`, `filter`, `forEach` and
// `reduce` is exposed on the interface.
class Vector extends _Base.default {
  constructor(Type, value = []) {
    super(value.map(entry => entry instanceof Type ? entry : new Type(entry)));
    this._length = void 0;
    this._Type = void 0;
    this._length = new _Length.default(value.length);
    this._Type = Type;
  }

  static with(Type) {
    return class extends Vector {
      constructor(value) {
        super(Type, value);
      }

    };
  }

  get Type() {
    return this._Type.name;
  }

  get length() {
    return this.raw.length;
  }

  byteLength() {
    return this.raw.reduce((total, raw) => {
      return total + raw.byteLength();
    }, this._length.byteLength());
  }

  filter(fn) {
    return this.raw.filter(fn);
  }

  find(fn) {
    return this.raw.find(fn);
  }

  forEach(fn) {
    return this.raw.forEach(fn);
  }

  fromJSON(input) {
    this.raw = input.map(input => new this._Type().fromJSON(input));
    return this;
  }

  fromU8a(input) {
    this._length.fromU8a(input);

    const length = this._length.toNumber();

    let offset = this._length.byteLength();

    this.raw = [];

    for (let index = 0; index < length; index++) {
      const raw = new this._Type().fromU8a(input.subarray(offset));
      this.raw.push(raw);
      offset += raw.byteLength();
    }

    return this;
  }

  get(index) {
    return this.raw[index];
  }

  map(fn) {
    return this.raw.map(fn);
  }

  push(item) {
    this.raw.push(item);

    this._length.setValue(this.raw.length);
  }

  reduce(fn, initial) {
    return this.raw.reduce(fn, initial);
  }

  toJSON() {
    return this.raw.map(entry => entry.toJSON());
  }

  toU8a(isBare) {
    return (0, _concat.default)(this._length.toU8a(isBare), ...this.raw.map(entry => entry.toU8a(isBare)));
  }

  toString() {
    const data = this.raw.map(entry => entry.toString());
    return `[${data.join(', ')}]`;
  }

}

exports.default = Vector;