"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.typeSplit = typeSplit;
exports.getVectorType = getVectorType;
exports.getTupleType = getTupleType;
exports.getType = getType;
exports.default = createType;

var _assert = _interopRequireDefault(require("@polkadot/util/assert"));

var Types = _interopRequireWildcard(require("../index"));

var _Tuple = _interopRequireDefault(require("./Tuple"));

var _Vector = _interopRequireDefault(require("./Vector"));

// Copyright 2017-2018 @polkadot/api-codec authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
// safely split a string on ', ' while taking care of any nested occurences
function typeSplit(type) {
  let tDepth = 0;
  let vDepth = 0;
  let start = 0;
  const result = [];

  for (let index = 0; index < type.length; index++) {
    switch (type[index]) {
      case ',':
        // we are not nested, add the type
        if (tDepth === 0 && vDepth === 0) {
          result.push(type.substr(start, index - start).trim());
          start = index + 1;
        }

        break;

      case '(':
        // inc tuple depth
        tDepth++;
        break;

      case ')':
        // dec tuple depth
        tDepth--;
        break;

      case '<':
        // inc vec depth
        vDepth++;
        break;

      case '>':
        // dec tuple depth
        vDepth--;
        break;

      default:
        break;
    }
  }

  (0, _assert.default)(tDepth === 0, `Invalid Tuple in ${type}`);
  (0, _assert.default)(vDepth === 0, `Invalid Vector in ${type}`); // the final leg of the journey

  result.push(type.substr(start, type.length - start).trim());
  return result;
} // Handle Vector types, i.e Vec<AccountId> or Vec<(AccountId, Balance)>


function getVectorType(type) {
  // l.debug(() => ['getVectorType', { type }]);
  (0, _assert.default)(type.substr(0, 4) === 'Vec<' && type[type.length - 1] === '>', `Expected Vec wrapped with <>`); // strip wrapping Vec<>

  const subType = type.substr(4, type.length - 5);
  return _Vector.default.with(getType(subType));
} // Handle tuple types, (u32, String, AccountId). It could be nested and wrapped in other
// types, i.e. (u32, Vec<(AccountId, Balance)>)


function getTupleType(type) {
  // l.debug(() => ['getTupleType', { type }]);
  (0, _assert.default)(type[0] === '(' && type[type.length - 1] === ')', `Expected tuple wrapped with ()`); // strip wrapping ()'s

  const innerTypes = typeSplit(type.substr(1, type.length - 2));
  return _Tuple.default.with(innerTypes.reduce((result, type, index) => {
    result[`entry${index}`] = getType(type);
    return result;
  }, {}));
} // Returns the type Class for construction


function getType(_type) {
  const type = _type.trim(); // l.debug(() => ['getType', { type }]);


  if (type[0] === '(') {
    return getTupleType(type);
  } else if (type.substr(0, 4) === 'Vec<') {
    return getVectorType(type);
  }

  const Type = Types[type];
  (0, _assert.default)(Type, `Unable to determine type from '${type}'`);
  return Type;
}

function createType(type, value) {
  // l.debug(() => ['createType', { type, value }]);
  const Type = getType(type);
  return new Type(value);
}